/**
 * \license This project is released under the BSD 2-clause license
 *
 * Copyright (C) 2014 Xevolver Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.
 *    
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
/* descriptorΥåѷ */

#include <stdio.h>
#include <stdlib.h>

#ifndef MKL_ASLI64_INCLUDED
#include "mkl_asli64.h"
#endif

  /* ñ1DFT C descriptorå */
Int64 DftiCreateDescriptor_s_1d(struct MKL2ASL_DESCRIPTOR_D **desc, const Int32 domain, const Int64 length)
{
  struct MKL2ASL_DESCRIPTOR_D *alloc_ptr;

  if ((alloc_ptr = (struct MKL2ASL_DESCRIPTOR_D *)malloc(sizeof(struct MKL2ASL_DESCRIPTOR_D)))
     == NULL)
  {
    fprintf(stderr,"Insufficient memory.\n");
    return MKL2ASL_MEMORY_ERROR;
  }

  alloc_ptr->commit_flag    = MKL2ASL_UNCOMMITTED;
  alloc_ptr->precision      = MKL2ASL_SINGLE;
  alloc_ptr->domain         = MKL2ASL_COMPLEX;
  alloc_ptr->dim            = 1;
  alloc_ptr->len_1d         = length;
  alloc_ptr->length         = &alloc_ptr->len_1d;
  alloc_ptr->placement_flag = MKL2ASL_INPLACE;
  *desc = alloc_ptr;

  return MKL2ASL_NO_ERROR;
}

  /* ñ¿DFT C descriptorå */
Int64 DftiCreateDescriptor_s_md(struct MKL2ASL_DESCRIPTOR_D **desc, const Int32 domain, const Int64 dim, Int64 *length)
{
  struct MKL2ASL_DESCRIPTOR_D *alloc_ptr;

  if ((alloc_ptr = (struct MKL2ASL_DESCRIPTOR_D *)malloc(sizeof(struct MKL2ASL_DESCRIPTOR_D)))
     == NULL)
  {
    fprintf(stderr,"Insufficient memory.\n");
    return MKL2ASL_MEMORY_ERROR;
  }

  alloc_ptr->commit_flag    = MKL2ASL_UNCOMMITTED;
  alloc_ptr->precision      = MKL2ASL_SINGLE;
  alloc_ptr->domain         = MKL2ASL_COMPLEX;
  alloc_ptr->dim            = dim;
  alloc_ptr->length         = length;
  alloc_ptr->placement_flag = MKL2ASL_INPLACE;
  *desc = alloc_ptr;

  return MKL2ASL_NO_ERROR;
}

  /* 1DFT C descriptorå */
Int64 DftiCreateDescriptor_d_1d(struct MKL2ASL_DESCRIPTOR_D **desc, const Int32 domain, const Int64 length)
{
  struct MKL2ASL_DESCRIPTOR_D *alloc_ptr;

  if ((alloc_ptr = (struct MKL2ASL_DESCRIPTOR_D *)malloc(sizeof(struct MKL2ASL_DESCRIPTOR_D)))
     == NULL)
  {
    fprintf(stderr,"Insufficient memory.\n");
    return MKL2ASL_MEMORY_ERROR;
  }

  alloc_ptr->commit_flag    = MKL2ASL_UNCOMMITTED;
  alloc_ptr->precision      = MKL2ASL_DOUBLE;
  alloc_ptr->domain         = MKL2ASL_COMPLEX;
  alloc_ptr->dim            = 1;
  alloc_ptr->len_1d         = length;
  alloc_ptr->length         = &alloc_ptr->len_1d;
  alloc_ptr->placement_flag = MKL2ASL_INPLACE;
  *desc = alloc_ptr;

  return MKL2ASL_NO_ERROR;
}

  /* ¿DFT C descriptorå */
Int64 DftiCreateDescriptor_d_md(struct MKL2ASL_DESCRIPTOR_D **desc, const Int32 domain, const Int64 dim, Int64 *length)
{
  struct MKL2ASL_DESCRIPTOR_D *alloc_ptr;

  if ((alloc_ptr = (struct MKL2ASL_DESCRIPTOR_D *)malloc(sizeof(struct MKL2ASL_DESCRIPTOR_D)))
     == NULL)
  {
    fprintf(stderr,"Insufficient memory.\n");
    return MKL2ASL_MEMORY_ERROR;
  }

  alloc_ptr->commit_flag    = MKL2ASL_UNCOMMITTED;
  alloc_ptr->precision      = MKL2ASL_DOUBLE;
  alloc_ptr->domain         = MKL2ASL_COMPLEX;
  alloc_ptr->dim            = dim;
  alloc_ptr->length         = length;
  alloc_ptr->placement_flag = MKL2ASL_INPLACE;
  *desc = alloc_ptr;

  return MKL2ASL_NO_ERROR;
}

Int64 DftiCreateDescriptor(struct MKL2ASL_DESCRIPTOR_D **desc, const Int32 precision, const Int32 domain,
                           const Int64 dim, ...)
{
  return MKL2ASL_INVALID_CONFIGURATION;
}
